library ieee;                   
library ieee, std, work;              
use ieee.std_logic_1164.all;    
use work.all;                   
use std.textio.all;             
use ieee.std_logic_textio.all;  
use ieee.std_logic_arith.all;   
use ieee.std_logic_unsigned.all;

use work.xhdl_std_logic.all;
use work.xhdl_misc.all;
use work.coreparameters.all;

ENTITY g4_dp_ext_mem IS
   GENERIC (
      -- Memory parameters
      RAM_RW                         :  integer := 18;    
      RAM_WW                         :  integer := 18;    
      RAM_WD                         :  integer := 10;    
      RAM_RD                         :  integer := 10;    
      RAM_ADDRESS_END                :  integer := 1024;    
      WRITE_CLK                      :  integer := 1;    
      READ_CLK                       :  integer := 1;    
      SYNC                           :  integer := 1;    
      WRITE_ENABLE                   :  integer := 0;    
      READ_ENABLE                    :  integer := 0;    
      RESET_POLARITY                 :  integer := 0);    
   PORT (
      -- local inputs

      clk                     : IN std_logic;   
      wclk                    : IN std_logic;   
      rclk                    : IN std_logic;   
      rst_n                   : IN std_logic;   
      -- local inputs - memory functional bus

      waddr                   : IN std_logic_vector(RAM_WD - 1 DOWNTO 0);   
      raddr                   : IN std_logic_vector(RAM_RD - 1 DOWNTO 0);   
      data                    : IN std_logic_vector(RAM_WW - 1 DOWNTO 0);   
      we                      : IN std_logic;   
      re                      : IN std_logic;   
      --OUTPUTS
      q                       : OUT std_logic_vector(RAM_RW - 1 DOWNTO 0));   
END ENTITY g4_dp_ext_mem;

ARCHITECTURE translated OF g4_dp_ext_mem IS

   TYPE xhdl_2 IS ARRAY (RAM_ADDRESS_END - 1 DOWNTO 0) OF 
   std_logic_vector(RAM_WW - 1 DOWNTO 0);

   SIGNAL clk_int                  :  std_logic;   
   SIGNAL wclk_int                 :  std_logic;   
   SIGNAL rclk_int                 :  std_logic;   
   SIGNAL we_int                   :  std_logic;   
   SIGNAL re_int                   :  std_logic;   
   SIGNAL waddr_int                :  std_logic_vector(RAM_WD - 1 DOWNTO 0);   
   SIGNAL rst_int                  :  std_logic;   
   SIGNAL wdone                    :  std_logic;   
   SIGNAL temp                     :  std_logic;   
   -- ** MEM DECLARATION **
   SIGNAL MEM                      :  xhdl_2;   
   SIGNAL i                        :  integer;   
   SIGNAL temp_xhdl3               :  std_logic;   
   SIGNAL temp_xhdl4               :  std_logic;   
   SIGNAL temp_xhdl5               :  std_logic;   
   SIGNAL temp_xhdl6               :  std_logic;   
   SIGNAL temp_xhdl7               :  std_logic;   
   SIGNAL temp_xhdl8               :  std_logic;   
   SIGNAL temp_xhdl9               :  std_logic;   
   SIGNAL temp_xhdl10              :  std_logic;   
   SIGNAL q_xhdl1                  :  std_logic_vector(RAM_RW - 1 DOWNTO 0);   

BEGIN
   --q <= q_xhdl1;
   temp_xhdl3 <= clk WHEN WRITE_CLK /= 0 ELSE NOT clk;
   clk_int <= temp_xhdl3 ;
   temp_xhdl4 <= wclk WHEN WRITE_CLK /= 0 ELSE NOT wclk;
   wclk_int <= temp_xhdl4 ;
   temp_xhdl5 <= rclk WHEN READ_CLK /= 0 ELSE NOT rclk;
   rclk_int <= temp_xhdl5 ;
   temp_xhdl6 <= wclk WHEN WRITE_CLK /= 0 ELSE NOT wclk;
   temp_xhdl7 <= clk_int WHEN SYNC /= 0 ELSE (temp_xhdl6);
   wclk_int <= temp_xhdl7 ;
   temp_xhdl8 <= rclk WHEN READ_CLK /= 0 ELSE NOT rclk;
   temp_xhdl9 <= clk_int WHEN SYNC /= 0 ELSE (temp_xhdl8);
   rclk_int <= temp_xhdl9 ;
   we_int <= we ;
   re_int <= re ;
   waddr_int <= waddr after 1 ns;
   temp_xhdl10 <= NOT rst_n WHEN RESET_POLARITY /= 0 ELSE rst_n;
   rst_int <= temp_xhdl10 ;

   PROCESS
      VARIABLE xhdl_initial : BOOLEAN := TRUE;
      VARIABLE q_xhdl1_xhdl11  : std_logic_vector(RAM_RW - 1 DOWNTO 0);
      VARIABLE wdone_xhdl12  : std_logic;
      VARIABLE temp_xhdl13  : std_logic;
   BEGIN
      IF (xhdl_initial) THEN
         q_xhdl1_xhdl11 := (OTHERS => '0');    
         wdone_xhdl12 := '0';    
         temp_xhdl13 := '0';    
         q_xhdl1 <= q_xhdl1_xhdl11;
         wdone <= wdone_xhdl12;
         temp <= temp_xhdl13;
         xhdl_initial := FALSE;
      ELSE
         WAIT;
      END IF;
   END PROCESS;

   PROCESS(we_int,wclk_int)
   BEGIN
        IF (we_int = '1') THEN
           MEM(to_integer(waddr_int)) <= data;    
        END IF;
   END PROCESS;

   PROCESS (clk_int, rst_int)
 VARIABLE q1_xhdl1  : std_logic_vector(RAM_RW - 1 DOWNTO 0);
   BEGIN
      IF (NOT rst_int = '1') THEN
      ELSIF (clk_int'EVENT AND clk_int = '1') THEN
         IF (re_int = '1') THEN
            q1_xhdl1 := MEM(to_integer(raddr));    
         END IF;
      END IF;
      q <= q1_xhdl1;
   END PROCESS;

END ARCHITECTURE translated;
