---------------------------------------------------------------------------------------------------
-- DESIGN RCU FIRM WARE V.2.0
-- FILE NAME   : ABS_CNT.VHD
-- DESIGN TEAM : Luciano Musa, Changzhou Xiang , Attiq ur Rehman
-- DESCRIPTION : All control Signal Active High, ASYNCHRONOUS RESET AND CLEAR
-- CLOCK DOMAIN: RCU CLOCK
-- PURPOSE     : ALTRO BUS SIGNAL COUNTER,COUNT NUMBER OF CSTB/TRSF/ACK/DSTB ASSERT TIMES(COUNT
--               SIGNAL RISING EDGE)
--           
-- AUTHORS     : Changzhou.Xiang (CERN-PH/ED)
--
-- UPDATE TIME : 28.05.08
-- NOTE        : Change dstb/trsf counters bit width.
--
-- UPDATE TIME : 15.05.08
-- NOTE        : Change dstb/trsf/ack/cstb counters bit width.

-- UPDATE TIME : 14.05.08
-- NOTE        : Correct dstba_cnt/dstbb_cnt mistake, use dstb clock dstb counters.s
---------------------------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity ABS_CNT is	 	
	port( 
		clk			: in  std_logic;	-- 40 MHz clock input
		rst			: in  std_logic;	-- reset signal
		clr			: in  std_logic;	-- clear counter
		
		dstb_a		: in  std_logic;	-- dstb of branch a
		dstb_b		: in  std_logic;	-- dstb of branch b
		trsf_a		: in  std_logic;	-- trsf of branch a
		trsf_b		: in  std_logic;	-- trsf of branch b
		ack_a		: in  std_logic;	-- ack of branch a
		ack_b		: in  std_logic;	-- ack of branch b
		cstb_a		: in  std_logic;	-- cstb of branch a
		cstb_b		: in  std_logic;	-- cstb of branch b
		
		dstba_cnt	: out std_logic_vector(19 downto 0);	-- number of dstb negedge for branch a  
		dstbb_cnt	: out std_logic_vector(19 downto 0);	-- number of dstb negedge for branch b
		trsfa_cnt	: out std_logic_vector(11 downto 0);	-- number of trsf posedge for branch a  
		trsfb_cnt	: out std_logic_vector(11 downto 0);	-- number of trsf posedge for branch b 
		acka_cnt	: out std_logic_vector(11 downto 0);	-- number of ack posedge for branch a  
		ackb_cnt	: out std_logic_vector(11 downto 0);	-- number of ack posedge for branch b 
		cstba_cnt	: out std_logic_vector(11 downto 0);	-- number of cstb posedge for branch a  
		cstbb_cnt	: out std_logic_vector(11 downto 0)		-- number of cstb posedge for branch b            	
	);           
end ABS_CNT; 
			   	
architecture A_ABSCNT of ABS_CNT is	
	
	
	COMPONENT CNT12
	PORT(
		clk : IN std_logic;
		rst : IN std_logic;
		clr : IN std_logic;
		en : IN std_logic;          
		cnt_out : OUT std_logic_vector(11 downto 0)
		);
	END COMPONENT;
	
	COMPONENT CNT20
	PORT(
		clk : IN std_logic;
		rst : IN std_logic;
		clr : IN std_logic;
		en : IN std_logic;          
		cnt_out : OUT std_logic_vector(19 downto 0)
		);
	END COMPONENT;
	
	COMPONENT POSEDGE_PULSE
	PORT(
		clk : IN std_logic;
		bit_in : IN std_logic;          
		bit_out : OUT std_logic
		);
	END COMPONENT;
	

	-- signal dstba_counter:std_logic_vector (11 downto 0);
	-- signal dstbb_counter:std_logic_vector (11 downto 0);
	
	
	signal trsf_pulse_a		:std_logic;		-- trsf pulse for branch A
	signal trsf_pulse_b		:std_logic;		-- trsf pulse for branch B
	signal ack_pulse_a		:std_logic;		-- ack pulse for branch A
	signal ack_pulse_b		:std_logic;		-- ack pulse for branch B
	signal cstb_pulse_a		:std_logic;		-- cstb pulse for branch A
	signal cstb_pulse_b		:std_logic;		-- cstb pulse for branch B
		
begin

	
	--------------U1_DSTBCNTA--------------
	U1_DSTBCNTA : CNT20 PORT MAP(
		clk => dstb_a,
		rst => rst,
		clr => clr,
		en => '1',
		cnt_out => dstba_cnt
	);
	
	--------------U2_DSTBCNTB--------------
	U2_DSTBCNTB : CNT20 PORT MAP(
		clk => dstb_b,
		rst => rst,
		clr => clr,
		en => '1',
		cnt_out => dstbb_cnt
	);
	
	--------------U3_TRSFPULSEA--------------
	U3_TRSFPULSEA : POSEDGE_PULSE PORT MAP(
		clk => clk,
		bit_in => trsf_a,
		bit_out => trsf_pulse_a
	);
	--------------U3_TRSFCNTA--------------
	U3_TRSFCNTA : CNT12 PORT MAP(
		clk => clk,
		rst => rst,
		clr => clr,
		en => trsf_pulse_a,
		cnt_out => trsfa_cnt
	);
	
	--------------U4_TRSFPULSEB--------------
	U4_TRSFPULSEB : POSEDGE_PULSE PORT MAP(
		clk => clk,
		bit_in => trsf_b,
		bit_out => trsf_pulse_b
	);
	--------------U4_TRSFCNTB--------------
	U4_TRSFCNTB : CNT12 PORT MAP(
		clk => clk,
		rst => rst,
		clr => clr,
		en => trsf_pulse_b,
		cnt_out => trsfb_cnt
	);
	
	--------------U5_ACKPULSEA--------------
	U5_ACKPULSEA : POSEDGE_PULSE PORT MAP(
		clk => clk,
		bit_in => ack_a,
		bit_out => ack_pulse_a
	);
	--------------U5_ACKCNTA--------------
	U5_ACKCNTA : CNT12 PORT MAP(
		clk => clk,
		rst => rst,
		clr => clr,
		en => ack_pulse_a,
		cnt_out => acka_cnt
	);
	
	--------------U6_ACKPULSEB--------------
	U6_ACKPULSEB : POSEDGE_PULSE PORT MAP(
		clk => clk,
		bit_in => ack_b,
		bit_out => ack_pulse_b
	);
	--------------U6_ACKCNTB--------------
	U6_ACKCNTB : CNT12 PORT MAP(
		clk => clk,
		rst => rst,
		clr => clr,
		en => ack_pulse_b,
		cnt_out => ackb_cnt
	);
	
	
	--------------U7_CSTBPULSEA--------------
	U7_CSTBPULSEA : POSEDGE_PULSE PORT MAP(
		clk => clk,
		bit_in => cstb_a,
		bit_out => cstb_pulse_a
	);
	--------------U7_CSTBCNTA--------------
	U7_CSTBCNTA : CNT12 PORT MAP(
		clk => clk,
		rst => rst,
		clr => clr,
		en => cstb_pulse_a,
		cnt_out => cstba_cnt
	);
	
	--------------U8_CSTBPULSEB--------------
	U8_CSTBPULSEB : POSEDGE_PULSE PORT MAP(
		clk => clk,
		bit_in => cstb_b,
		bit_out => cstb_pulse_b
	);
	--------------U8_CSTBCNTB--------------
	U8_CSTBCNTB : CNT12 PORT MAP(
		clk => clk,
		rst => rst,
		clr => clr,
		en => cstb_pulse_b,
		cnt_out => cstbb_cnt
	);
	
	-- -- --------------U3_TRSF_A--------------
	-- -- U3_TRSFPULSEA : POSEDGE_PULSE PORT MAP(
		-- -- clk => clk,
		-- -- bit_in => trsf_a,
		-- -- bit_out => trsf_pulse_a
	-- -- );
	
		
	
	-- -- --------------U3_TRSF_A--------------
	-- -- U3_TRSF_A : SIGNAL_CNT PORT MAP(
		-- -- clk => clk,
		-- -- rst => rst,
		-- -- clr => clr,
		-- -- abs_in => trsf_a,
		-- -- cnt_out => trsfa_cnt
	-- -- );
		
	-- -- --------------U4_TRSF_B--------------
	-- -- U4_TRSF_B : SIGNAL_CNT PORT MAP(
		-- -- clk => clk,
		-- -- rst => rst,
		-- -- clr => clr,
		-- -- abs_in => trsf_b,
		-- -- cnt_out => trsfb_cnt
	-- -- );
	
	
	-- --------------U5_ACK_A--------------
	-- U5_ACK_A : SIGNAL_CNT PORT MAP(
		-- clk => clk,
		-- rst => rst,
		-- clr => clr,
		-- abs_in => ack_a,
		-- cnt_out => acka_cnt
	-- );
	
	-- --------------U6_ACK_B--------------
	-- U6_ACK_B : SIGNAL_CNT PORT MAP(
		-- clk => clk,
		-- rst => rst,
		-- clr => clr,
		-- abs_in => ack_b,
		-- cnt_out => ackb_cnt
	-- );
	
	
	-- --------------U7_CSTB_A--------------
	-- U7_CSTB_A : SIGNAL_CNT PORT MAP(
		-- clk => clk,
		-- rst => rst,
		-- clr => clr,
		-- abs_in => cstb_a,
		-- cnt_out => cstba_cnt
	-- );
	
	-- --------------U8_CSTB_B--------------
	-- U8_CSTB_B : SIGNAL_CNT PORT MAP(
		-- clk => clk,
		-- rst => rst,
		-- clr => clr,
		-- abs_in => cstb_b,
		-- cnt_out => cstbb_cnt
	-- );
end;
		