---------------------------------------------------------------------------------------------------
-- DESIGN RCU FIRM WARE V.2.0
-- FILE NAME   : ADDR_DECODER.V
-- DESIGN TEAM : Luciano Musa, Changzhou Xiang , Attiq ur Rehman
-- DESCRIPTION : All control Signal Active High, Data RD/WR on rising edge,ASYNCHRONOUS RESET AND CLEAR
-- CLOCK DOMAIN: RCU CLOCK
-- PURPOSE     : RCU ADDRESS ENABLE SIGNAL
--           
-- AUTHORS     : Changzhou.Xiang (CERN-PH/ED)
--
-- UPDATE TIME : 11.05.08
-- NOTE        : Change RCU register address range, it is 0x5100 to 0x51FF now!
-------------------------------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

library WORK;
use work.rcu_addr_set.all;

entity ADDR_DECODER is
    Port(
		rcu_add			: in  STD_LOGIC_VECTOR(15 downto 5);	-- ADDRESS FROM RCU BUS 
				
		rcu_im_en		: out std_logic;	-- INSTRUCTION MEMORY ENABLE
		rcu_rolm_en		: out std_logic;	-- READOUT-LIST MEMORY ENABLE
		rcu_rm_en		: out std_logic;	-- RESULT MEMORY ENABLE
		rcu_ttcrx_en	: out std_logic;	-- RESULT MEMORY ENABLE
		rcu_hlm_en 		: out std_logic;	-- HIT-LIST MEMORY ENABLE
		rcu_reg_en		: out std_logic;	-- RCU REGISTER ENABLE
		rcu_cmd_en		: out std_logic;	-- RCU COMMAND ENABLE
		sel 			: out std_logic_vector(7 downto 0)		-- ALTRO DATA MEMORY ENABLE
	);
end ADDR_DECODER;

architecture A_ADDRDECODER of ADDR_DECODER is   
begin 
	rcu_im_en <= '1' when rcu_add(15 downto 12)=addr_im else '0';
	
	rcu_rolm_en <= '1' when rcu_add(15 downto 12)=addr_rolm else '0';
	
	rcu_rm_en <= '1' when rcu_add(15 downto 13)=addr_rm else '0';
	
	rcu_ttcrx_en <= '1' when rcu_add(15 downto 12)=addr_ttcrx else '0';
	
	rcu_hlm_en <= '1' when rcu_add(15 downto 7)=addr_hlm else '0';
	
	rcu_reg_en <= '1' when rcu_add(15 downto 8)=addr_reg else '0';
	rcu_cmd_en <= '1' when rcu_add(15 downto 5)=addr_cmd else '0';
	
	sel(0) <= '1' when rcu_add(15 downto 8)=addr_dm_0 else '0';
	sel(1) <= '1' when rcu_add(15 downto 8)=addr_dm_1 else '0';
	sel(2) <= '1' when rcu_add(15 downto 8)=addr_dm_2 else '0';
	sel(3) <= '1' when rcu_add(15 downto 8)=addr_dm_3 else '0';
	sel(4) <= '1' when rcu_add(15 downto 8)=addr_dm_4 else '0';
	sel(5) <= '1' when rcu_add(15 downto 8)=addr_dm_5 else '0';
	sel(6) <= '1' when rcu_add(15 downto 8)=addr_dm_6 else '0';
	sel(7) <= '1' when rcu_add(15 downto 8)=addr_dm_7 else '0';
	
end;

