-------------------------------------------------------------------------------
-- Title      : Broadcast Message Decoder MOdule
-- Project    : RCU Trigger Receiver
-------------------------------------------------------------------------------
-- File             : $RCSfile: broadcast_msg_decoder.vhd,v $
-- Last edited by   : $Author: alme $
-- Last update      : $Date: 2007-12-10 15:22:02 $
-- Current Revision : $Revision: 1.6 $
-------------------------------------------------------------------------------
-- Description: Decodes broadcast messages to event and bunch reset.
-------------------------------------------------------------------------------
-- Revision History : 
-- http://www.ift.uib.no/kjekscgi-bin/viewcvs.cgi/vhdlcvs/trigger_receiver/
-------------------------------------------------------------------------------
--
-- This file is property of and copyright by the Instrumentation and 
-- Electronics Section, Dep. of Physics and Technology
-- University of Bergen, Norway, 2005
-- This file has been written by Johan Alme
-- Johan.Alme@ift.uib.no
--
-- Permission to use, copy, modify and distribute this firmware and its  
-- documentation strictly for non-commercial purposes is hereby granted  
-- without fee, provided that the above copyright notice appears in all  
-- copies and that both the copyright notice and this permission notice  
-- appear in the supporting documentation. The authors make no claims    
-- about the suitability of this firmware for any purpose. It is         
-- provided "as is" without express or implied warranty.                 
--
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

library work;
use work.trigger_receiver_pkg.all;


entity broadcast_message_decoder is
  port(
    clk            : in  std_logic;
    reset_n        : in  std_logic;
    brcdata        : in  std_logic_vector(7 downto 0);
    brcdata_ready  : in  std_logic;
    bunchCnt_reset : out std_logic;
    eventCnt_reset : out std_logic;--not used but included anyway
    cal_pre_pulse  : out std_logic
  );
end broadcast_message_decoder;

architecture behave of broadcast_message_decoder is
  -- delaying the internal signals
  signal cal_pre_pulseR       : std_logic:= '0';
  signal bunchCnt_reset_iR    : std_logic:= '0';
  signal bunchCnt_reset_iRR   : std_logic:= '0';
  signal eventCnt_reset_iR    : std_logic:= '0';
  signal eventCnt_reset_iRR   : std_logic:= '0';
  
  alias user_message     : std_logic_vector(1 downto 0) is brcdata(7 downto 6); 
  alias system_message   : std_logic_vector(3 downto 0) is brcdata(5 downto 2);
  alias cal_pre_pulse_i  : std_logic is brcdata(2);--ref conversation with Pedja Jovanivic Nov 07 could ANY bit be used as pre pulse
  alias eventCnt_reset_i : std_logic is brcdata(1);
  alias bunchCnt_reset_i : std_logic is brcdata(0);
begin
  
  p_decoder: process(clk, reset_n)
  begin
    if (reset_n = '0') then
      bunchCnt_reset_iR <= '0';
      eventCnt_reset_iR <= '0';
      cal_pre_pulseR    <= '0';
    elsif rising_edge(clk) then
      bunchCnt_reset_iR <= '0';
      eventCnt_reset_iR <= '0';
      cal_pre_pulseR    <= '0';
   
      if (brcdata_ready='1') then
        bunchCnt_reset_iR <= bunchCnt_reset_i;
        eventCnt_reset_iR <= eventCnt_reset_i;
        cal_pre_pulseR    <= cal_pre_pulse_i;
      end if;
    end if;
  end process p_decoder;

-- purpose: delaying the generated signals to match the timing of
--          the equivialent reset signals generated by the TTCrx chip.
p_delay : process(clk, reset_n)
begin
  if (reset_n = '0') then
    bunchCnt_reset_iRR  <= '0';
    eventCnt_reset_iRR  <= '0';
    bunchCnt_reset      <= '0';
    eventCnt_reset  	  <= '0';
    cal_pre_pulse       <= '0';
  elsif rising_edge(clk) then

    bunchCnt_reset_iRR <= bunchCnt_reset_iR;
    eventCnt_reset_iRR <= eventCnt_reset_iR;
    
    bunchCnt_reset     <= bunchCnt_reset_iRR;
    eventCnt_reset     <= eventCnt_reset_iRR;
    cal_pre_pulse      <= cal_pre_pulseR;  --delaying only 1 clk to match L0/L1a decoding.
  end if;
end process p_delay;

end behave;
