-------------------------------------------------------------------------------------
-- Title            : ALICE TPC RCU2 TTC tester top
-- Project          : ALICE TPC RCU2 TTC tester
-------------------------------------------------------------------------------------
-- File             : $HeadURL:  $
-- Last edited by   : $Author:  $
-- Author:          : Arild Velure
-- Last update      : $Date:  $
-- Current Revision : $Revision:  $
-- Company          : University of Bergen
-- Created          : 2013-06-28
-- Platform         : Libero v11.1
-- Device           : Microsemi SmartFusion2
-- Standard         : VHDL'02
-------------------------------------------------------------------------------------
-- Description:     This module is the top module for the TTC tester 
--                  
--                  
--                  
-------------------------------------------------------------------------------------
--
-- This file is property of and copyright by the Microelectronics group,
-- Dep. of Physics and Technology, University of Bergen, Norway, 2010.
--
-- Permission to use, copy, modify and distribute this firmware and its  
-- documentation strictly for non-commercial purposes is hereby granted  
-- without fee, provided that the above copyright notice appears in all  
-- copies and that both the copyright notice and this permission notice  
-- appear in the supporting documentation. The authors make no claims    
-- about the suitability of this software for any purpose. It is         
-- provided "as is" without express or implied warranty.                 
--
-------------------------------------------------------------------------------------
library ieee;                  
use ieee.std_logic_1164.all;

library work;
use work.all;

library SmartFusion2; -- Microsemi/Libero_v11.1/Designer/lib/modelsim/precompiled/vhdl/SmartFusion2
use SmartFusion2.all;

entity cdr_top is
  GENERIC (
      U_dly                          :  integer := 2;
      NUM_BUF                        : integer := 35);  -- 0.22ns delay per bufd on SF2
  port (--general signal   
	  rst 			 : in std_logic;       	       	         
	  tx_data_in     : in std_logic; -- Diff manchester data
	  tx_data_in_del : out std_logic; -- Buffer delayed data
	     --  receive side  side           
	  --rx_err_det:out std_logic;    
	  rx_clk_out     : out std_logic;    
	  rx_data_out    : out std_logic	         	        	        	             	       	       
);          
end cdr_top; 

architecture behave of cdr_top is    
     
component hp_cdr  
 port(                                        
      rst         : in std_logic;
      data_in     : in std_logic;
      data_in_del : in std_logic;
      rx_ena      : in std_logic;
      err_det     : out std_logic;      
      clk_out     : out std_logic;
      data_out    : out std_logic               
); 
end component;

component BUFD
  -- Port list
  port(
    A : in  std_logic;
    Y : out std_logic
  );
end component;


signal enc_data : std_logic;      -- After DM encoding
signal enc_data_del : std_logic;  -- DM encoded data after delay through buffer
signal dec_data : std_logic;      -- Decoded data
--signal clk_data : std_logic;      -- 80MHz
--signal clk_ddr : std_logic;       -- 160MHz
signal rx_ena : std_logic;        -- Unused
signal rx_error_det : std_logic;  -- Unused
signal buf_in : std_logic_vector(1 to NUM_BUF); -- Inputs to buffers as a vector for use in generate statement
signal buf_out : std_logic_vector(1 to NUM_BUF);-- Outputs from buffers as a vector for use in generate statement

begin
 enc_data <= tx_data_in;
 tx_data_in_del <= enc_data_del; -- debug output to pin
 rx_data_out <= dec_data; -- debug output to pin
 rx_ena <= '1';
 

gen_buf:
  for index in 1 to NUM_BUF generate
  BUFD_inst : BUFD
    port map( 
    -- Inputs
    A => buf_in(index),
    -- Outputs
    Y => buf_out(index) 
    );
  end generate;

gen_links:    
  for i in 1 to (NUM_BUF-1) generate
    buf_in(i+1) <= buf_out(i);
  end generate;

buf_in(1) <= enc_data;            -- Input to buffer train
enc_data_del <= buf_out(NUM_BUF); -- Output from buffer train

cdr: entity work.hp_cdr(hp_fig2) -- hp_fig2 for simple version, hp_fig4 for version with recovered dutycycle of clock, logical for HLD version of fig2
  generic map (
	U_dly => U_dly
	)
	port map(
   rst=>rst,
   data_in=>enc_data,
   data_in_del=>enc_data_del,
   rx_ena=>rx_ena,
   
   err_det=>rx_error_det,
   clk_out=>rx_clk_out,
   data_out=>dec_data
); 
end behave;  
	 		 