-------------------------------------------------------------------------------
-- Title      : Phase Check
-- Project    : RCU Trigger Receiver
-------------------------------------------------------------------------------
-- File             : $RCSfile: phase_check.vhd,v $
-- Last edited by   : $Author: alme $
-- Last update      : $Date: 2007/12/10 15:24:41 $
-- Current Revision : $Revision: 1.1 $
-------------------------------------------------------------------------------
-- Detects the phase of the trigger with respect to sampling clock.
-------------------------------------------------------------------------------
-- Revision History : 
-- http://www.ift.uib.no/kjekscgi-bin/viewcvs.cgi/vhdlcvs/trigger_receiver/
-------------------------------------------------------------------------------
--
-- This file is property of and copyright by the Instrumentation and 
-- Electronics Section, Dep. of Physics and Technology
-- University of Bergen, Norway, 2005
-- This file has been written by Johan Alme
-- Johan.Alme@ift.uib.no
--
-- Permission to use, copy, modify and distribute this firmware and its  
-- documentation strictly for non-commercial purposes is hereby granted  
-- without fee, provided that the above copyright notice appears in all  
-- copies and that both the copyright notice and this permission notice  
-- appear in the supporting documentation. The authors make no claims    
-- about the suitability of this firmware for any purpose. It is         
-- provided "as is" without express or implied warranty.                 
--
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.std_logic_unsigned.all;

entity phase_check is
  port(
    clk                 : in  std_logic; -- 40 MHz system clock
    sclk                : in  std_logic; -- sampling clock (2,5 - 10 MHz)
    reset_n             : in  std_logic; -- asynch reset, negative ploarity
    trigger             : in  std_logic; -- L0 or L1 trigger depending on PHOS or TPC mode
    clear               : in  std_logic; -- L2 trigger will clear stored value. 
    phase               : out std_logic_vector(4 downto 0)); -- counter giving phase of when trigger is received.
end phase_check;

architecture behave of phase_check is
  signal sclkR : std_logic;
  signal phase_cnt : std_logic_vector(4 downto 0);
begin
  
  p_sclk_edge_det : process(clk)
  begin
    if rising_edge(clk) then
      sclkR <= sclk;
    end if;    
  end process p_sclk_edge_det;
  
  p_counter : process(clk, reset_n)
  begin
    if (reset_n = '0') then
      phase_cnt <= (others => '0');
    elsif rising_edge(clk) then
      if (sclk = '1' and sclkR = '0') then
        phase_cnt <= (others => '0');
      else
        phase_cnt <= phase_cnt + 1;
      end if;
    end if;
  end process p_counter;
  
  p_sample: process(clk, reset_n)
  begin
    if (reset_n = '0') then
      phase <= (others => '0');     
    elsif rising_edge(clk) then
      if (clear = '1') then
        phase <= (others => '0');
      elsif (trigger = '1') then
        phase <= phase_cnt;
      end if;
    end if;
  end process p_sample;
  
end behave;
