---------------------------------------------------------------------------------------------------
-- DESIGN RCU FIRM WARE V.2.0
-- FILE NAME   : POSEDGE_PULSE.VHD
-- DESIGN TEAM : Luciano Musa, Changzhou Xiang , Attiq ur Rehman
-- DESCRIPTION : All control Signal Active High, ASYNCHRONOUS RESET AND CLEAR
-- CLOCK DOMAIN: RCU CLOCK
-- PURPOSE     : CONVERT SIGNAL RISING EDGE TO ONE CLCYE PULSE
--           
-- AUTHORS     : Changzhou.Xiang (CERN-PH/ED)
-- UPDATE TIME : 09.09.08
-- NOTE        : Using three dff generate pulse, can ignore glitch. 
--               But input should stay two clock cycle at least.
---------------------------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity POSEDGE_PULSE is	 	
	port( 
		clk			: in  std_logic;	-- 40 MHz clock input
		bit_in		: in  std_logic;	-- one bit width signal in, active high
		bit_out 	: out std_logic		-- output pulse signal           	
	);           
end POSEDGE_PULSE; 
			   	
architecture A_POSEDGE_PULSE of POSEDGE_PULSE is		
	signal dff_out1	: std_logic;
	signal dff_out2 : std_logic;
	signal dff_out3 : std_logic;
			
begin
	
	process(clk)  								
	begin
		if clk'event and clk='1' then
			dff_out1 <= bit_in;
		end if;
	end process;
	
	process(clk)  								
	begin
		if clk'event and clk='1' then
			dff_out2 <= dff_out1;
		end if;
	end process;
	
	process(clk)  								
	begin
		if clk'event and clk='1' then
			dff_out3 <= dff_out1 and dff_out2;
		end if;
	end process;
	
	bit_out <= dff_out1 and dff_out2 and (not dff_out3);
	
			
end;
		